@@ -233,9 +233,10 @@ struct rpcsvc_request {
          */
         rpcsvc_auth_data_t      verf;
 
-	/* Execute this request's actor function as a synctask? */
-	gf_boolean_t            synctask;
+	/* Execute this request's actor function in ownthread of program?*/
+	gf_boolean_t            ownthread;
 
+        gf_boolean_t            synctask;
         /* Container for a RPC program wanting to store a temp
          * request-specific item.
          */
@@ -246,6 +247,19 @@ struct rpcsvc_request {
 
         /* pointer to cached reply for use in DRC */
         drc_cached_op_t         *reply;
+
+        /* request queue in rpcsvc */
+        struct list_head         request_list;
+
+        /* Things passed to rpc layer from client */
+
+        /* @flags: Can be used for binary data passed in xdata to be
+           passed here instead */
+        unsigned int flags;
+
+        /* ctime: origin of time on the client side, ideally this is
+           the one we should consider for time */
+        struct timespec ctime;
 };
 
 #define rpcsvc_request_program(req) ((rpcsvc_program_t *)((req)->prog))
@@ -396,11 +410,24 @@ struct rpcsvc_program {
          */
         int                     min_auth;
 
-	/* Execute actor function as a synctask? */
-	gf_boolean_t            synctask;
+	/* Execute actor function in program's own thread? This will reduce */
+        /* the workload on poller threads */
+	gf_boolean_t            ownthread;
+        gf_boolean_t            alive;
 
+        gf_boolean_t            synctask;
         /* list member to link to list of registered services with rpcsvc */
         struct list_head        program;
+        struct list_head        request_queue;
+        pthread_mutex_t         queue_lock;
+        pthread_cond_t          queue_cond;
+        pthread_t               thread;
+        int                     threadcount;
+        /* eventthreadcount is just a readonly copy of the actual value
+         * owned by the event sub-system
+         * It is used to control the scaling of rpcsvc_request_handler threads
+         */
+        int                     eventthreadcount;
 };
 
 typedef struct rpcsvc_cbk_program {
@@ -412,7 +439,8 @@ typedef struct rpcsvc_cbk_program {
  * procedure handlers.
  */
 extern int
-rpcsvc_program_register (rpcsvc_t *svc, rpcsvc_program_t *program);
+rpcsvc_program_register (rpcsvc_t *svc, rpcsvc_program_t *program,
+                         gf_boolean_t add_to_head);
 
 extern int
 rpcsvc_program_unregister (rpcsvc_t *svc, rpcsvc_program_t *program);
@@ -434,6 +462,13 @@ rpcsvc_listener_destroy (rpcsvc_listener_t *listener);
 extern int
 rpcsvc_program_register_portmap (rpcsvc_program_t *newprog, uint32_t port);
 
+#ifdef IPV6_DEFAULT
+extern int
+rpcsvc_program_register_rpcbind6 (rpcsvc_program_t *newprog, uint32_t port);
+extern int
+rpcsvc_program_unregister_rpcbind6 (rpcsvc_program_t *newprog);
+#endif
+
 extern int
 rpcsvc_program_unregister_portmap (rpcsvc_program_t *newprog);
 
@@ -497,8 +532,8 @@ rpcsvc_auth_check (rpcsvc_t *svc, char *volname, char *ipaddr);
 extern int
 rpcsvc_transport_privport_check (rpcsvc_t *svc, char *volname, uint16_t port);
 
-#define rpcsvc_request_seterr(req, err)                 (req)->rpc_err = err
-#define rpcsvc_request_set_autherr(req, err)            (req)->auth_err = err
+#define rpcsvc_request_seterr(req, err)         ((req)->rpc_err = (int)(err))
+#define rpcsvc_request_set_autherr(req, err)    ((req)->auth_err = (int)(err))
 
 extern int rpcsvc_submit_vectors (rpcsvc_request_t *req);
 
@@ -612,4 +647,8 @@ rpcsvc_auth_array (rpcsvc_t *svc, char *volname, int *autharr, int arrlen);
 rpcsvc_vector_sizer
 rpcsvc_get_program_vector_sizer (rpcsvc_t *svc, uint32_t prognum,
                                  uint32_t progver, int procnum);
+extern int
+rpcsvc_ownthread_reconf (rpcsvc_t *svc, int new_eventthreadcount);
+
+void rpcsvc_autoscale_threads (glusterfs_ctx_t *ctx, rpcsvc_t *rpc, int incr);
 #endif
